import std.c.stdio;
import std.c.stdlib;
import std.string;

char* strdup(char *str)
{
	char* x = (char *)malloc(strlen(str) + 1);
	strcpy(x, str);
	return x;
}

enum { ht_num_primes = 28 };

uint ht_prime_list[ht_num_primes] = [
    53ul,         97ul,         193ul,       389ul,       769ul,
    1543ul,       3079ul,       6151ul,      12289ul,     24593ul,
    49157ul,      98317ul,      196613ul,    393241ul,    786433ul,
    1572869ul,    3145739ul,    6291469ul,   12582917ul,  25165843ul,
    50331653ul,   100663319ul,  201326611ul, 402653189ul, 805306457ul, 
    1610612741ul, 3221225473ul, 4294967291ul
];

struct ht_node {
    char *key;
    int val;
    ht_node *next;
};

struct ht_ht {
    int sz;
    ht_node **tbl;
    int iter_index;
    ht_node *iter_next;
    int items;
};

int ht_val(ht_node *node) {
    return(node.val);
}

char *ht_key(ht_node *node) {
    return(node.key);
}

int ht_hashcode(ht_ht *ht, char *key) {
    uint val = 0;
    for (; *key; ++key) val = 5 * val + *key;
    return(val % ht.sz);
}

ht_node *ht_node_create(char *key) {
    char *newkey;
    ht_node* node;
    if ((node = (ht_node *)malloc(ht_node.sizeof)) == null) {
	perror("malloc ht_node");
	exit(1);
    }
    if ((newkey = (char *)strdup(key)) == null) {
	perror("strdup newkey");
	exit(1);
    }
    node.key = newkey;
    node.val = 0;
    node.next = (ht_node *)null;
    return(node);
}

ht_ht *ht_create(int sz) {
    int i = 0;
    ht_ht* ht = (ht_ht *)malloc(ht_ht.sizeof);
    while (ht_prime_list[i] < sz) { i++; }
    ht.sz = ht_prime_list[i];
    ht.tbl = (ht_node **)calloc(ht.sz, (ht_node *).sizeof);
    ht.iter_index = 0;
    ht.iter_next = null;
    ht.items = 0;
    return(ht);
}

void ht_destroy(ht_ht *ht) {
    ht_node* cur, next;
    int i;
    for (i=0; i<ht.sz; i++) {
	next = ht.tbl[i];
	while (next) {
	    cur = next;
	    next = next.next;
	    free(cur.key);
	    free(cur);
	}
    }
    free(ht.tbl);
    free(ht);
}

ht_node *ht_find(ht_ht *ht, char *key) {
    int hash_code = ht_hashcode(ht, key);
    ht_node *node = ht.tbl[hash_code];
    while (node) {
	if (strcmp(key, node.key) == 0) return(node);
	node = node.next;
    }
    return((ht_node *)null);
}

ht_node *ht_find_new(ht_ht *ht, char *key) {
    int hash_code = ht_hashcode(ht, key);
    ht_node* prev = null, node = ht.tbl[hash_code];
    while (node) {
	if (strcmp(key, node.key) == 0) return(node);
	prev = node;
	node = node.next;
    }
    ht.items++;
    if (prev) {
	return(prev.next = ht_node_create(key));
    } else {
	return(ht.tbl[hash_code] = ht_node_create(key));
    }
}

/*
 *  Hash Table iterator data/functions
 */
ht_node *ht_next(ht_ht *ht) {
    uint index;
    ht_node *node = ht.iter_next;
    if (node) {
	ht.iter_next = node.next;
	return(node);
    } else {
	while (ht.iter_index < ht.sz) {
	    index = ht.iter_index++;
	    if (ht.tbl[index]) {
		ht.iter_next = ht.tbl[index].next;
		return(ht.tbl[index]);
	    }
	}
    }
    return((ht_node *)null);
}

ht_node *ht_first(ht_ht *ht) {
    ht.iter_index = 0;
    ht.iter_next = (ht_node *)null;
    return(ht_next(ht));
}

int ht_count(ht_ht *ht) {
    return(ht.items);
}

int main(char argv[][]) {
    int i, c=0, n = ((argv.length == 2) ? atoi(argv[1]) : 1);
    char buf[32];

    ht_ht *ht = ht_create(n);

    for (i=1; i<=n; i++) {
    sprintf(buf, "%x", i);
    (ht_find_new(ht, buf)).val = i;
    }

    for (i=n; i>0; i--) {
    sprintf(buf, "%d", i);
    if (ht_find(ht, buf)) c++;
    }

    ht_destroy(ht);

    printf("%d\n", c);
    return(0);
}